@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2010 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2010 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -26,6 +26,7 @@ TransferTimeout = 30 * 1000
 PongTimeout = 60 * 1000
 PingInterval = 5 * 1000
 SeparatorToken = '|||'
+SeparatorToken_b = b'|||'
 
 
 class Connection(QTcpSocket):
@@ -262,8 +263,8 @@ class Connection(QTcpSocket):
             self.abort()
             return
         
-        self.write("{0}{1}1{1}p".format(
-            Connection.ProtocolPing, SeparatorToken))
+        self.write(QByteArray("{0}{1}1{1}p".format(
+            Connection.ProtocolPing, SeparatorToken).encode("utf-8")))
     
     def __sendGreetingMessage(self):
         """
@@ -293,7 +294,7 @@ class Connection(QTcpSocket):
         
         while self.bytesAvailable() and self.__buffer.size() < maxSize:
             self.__buffer.append(self.read(1))
-            if self.__buffer.endsWith(SeparatorToken):
+            if self.__buffer.endsWith(SeparatorToken_b):
                 break
         
         return self.__buffer.size() - numBytesBeforeRead
@@ -306,10 +307,10 @@ class Connection(QTcpSocket):
         """
         if self.bytesAvailable() <= 0 or \
            self.__readDataIntoBuffer() <= 0 or \
-           not self.__buffer.endsWith(SeparatorToken):
+           not self.__buffer.endsWith(SeparatorToken_b):
             return 0
         
-        self.__buffer.chop(len(SeparatorToken))
+        self.__buffer.chop(len(SeparatorToken_b))
         number = self.__buffer.toInt()[0]
         self.__buffer.clear()
         return number
@@ -329,19 +330,20 @@ class Connection(QTcpSocket):
             return False
         
         self.__buffer.chop(len(SeparatorToken))
-        if self.__buffer == Connection.ProtocolPing:
+        protocolHeader = str(self.__buffer, encoding="utf-8")
+        if protocolHeader == Connection.ProtocolPing:
             self.__currentDataType = Connection.Ping
-        elif self.__buffer == Connection.ProtocolPong:
+        elif protocolHeader == Connection.ProtocolPong:
             self.__currentDataType = Connection.Pong
-        elif self.__buffer == Connection.ProtocolMessage:
+        elif protocolHeader == Connection.ProtocolMessage:
             self.__currentDataType = Connection.PlainText
-        elif self.__buffer == Connection.ProtocolGreeting:
+        elif protocolHeader == Connection.ProtocolGreeting:
             self.__currentDataType = Connection.Greeting
-        elif self.__buffer == Connection.ProtocolGetParticipants:
+        elif protocolHeader == Connection.ProtocolGetParticipants:
             self.__currentDataType = Connection.GetParticipants
-        elif self.__buffer == Connection.ProtocolParticipants:
+        elif protocolHeader == Connection.ProtocolParticipants:
             self.__currentDataType = Connection.Participants
-        elif self.__buffer == Connection.ProtocolEditor:
+        elif protocolHeader == Connection.ProtocolEditor:
             self.__currentDataType = Connection.Editor
         else:
             self.__currentDataType = Connection.Undefined
@@ -388,8 +390,8 @@ class Connection(QTcpSocket):
             self.newMessage.emit(
                 self.__username, str(self.__buffer, encoding="utf-8"))
         elif self.__currentDataType == Connection.Ping:
-            self.write("{0}{1}1{1}p".format(
-                Connection.ProtocolPong, SeparatorToken))
+            self.write(QByteArray("{0}{1}1{1}p".format(
+                Connection.ProtocolPong, SeparatorToken).encode("utf-8")))
         elif self.__currentDataType == Connection.Pong:
             self.__pongTime.restart()
         elif self.__currentDataType == Connection.GetParticipants:
@@ -414,10 +416,11 @@ class Connection(QTcpSocket):
         """
         Public method to request a list of participants.
         """
-        self.write(
+        self.write(QByteArray(
             "{0}{1}1{1}l".format(
-                Connection.ProtocolGetParticipants, SeparatorToken)
-        )
+                Connection.ProtocolGetParticipants, SeparatorToken
+            ).encode("utf-8")
+        ))
     
     def sendParticipants(self, participants):
         """
