@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2009 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2009 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -404,9 +404,11 @@ class HistoryMenu(E5ModelMenu):
         index = 0
         for tab in self.__tabWidget.closedTabsManager().allClosedTabs():
             title = fm.elidedText(tab.title, Qt.ElideRight, maxWidth)
-            self.__closedTabsMenu.addAction(
-                Helpviewer.HelpWindow.HelpWindow.icon(tab.url), title,
-                self.__tabWidget.restoreClosedTab).setData(index)
+            act = self.__closedTabsMenu.addAction(
+                Helpviewer.HelpWindow.HelpWindow.icon(tab.url), title)
+            act.setData(index)
+            act.triggered.connect(
+                lambda: self.__tabWidget.restoreClosedTab(act))
             index += 1
         self.__closedTabsMenu.addSeparator()
         self.__closedTabsMenu.addAction(
