@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2010 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2010 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 # Original (c) 2005 Divmod, Inc.  See __init__.py file for details
 #
@@ -486,5 +486,36 @@ class AssertTuple(Message):
     message_id = 'F28'
     message = 'assertion is always true, perhaps remove parentheses?'
 
+
+class ForwardAnnotationSyntaxError(Message):
+    """
+    Class defining the "forward annotation syntax error" message.
+    
+    Found a syntax error in forward annotation.
+    """
+    message_id = 'F29'
+    message = 'syntax error in forward annotation %r'
+
+    def __init__(self, filename, loc, annotation):
+        """
+        Constructor
+        
+        @param filename name of the file (string)
+        @param loc location of the issue
+        @param annotation erroneous forward annotation (string)
+        """
+        Message.__init__(self, filename, loc)
+        self.message_args = (annotation,)
+
+
+class RaiseNotImplemented(Message):
+    """
+    Class defining the "raise not implemented" message.
+    
+    Use NotImplementedError instead of NotImplemented.
+    """
+    message_id = 'F30'
+    message = "'raise NotImplemented' should be 'raise NotImplementedError'"
+
 #
 # eflag: noqa = M702
