@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2014 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2014 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -39,6 +39,10 @@ class TranslatorPage(ConfigurationPageBase, Ui_TranslatorPage):
         self.__plugin = plugin
         self.__enableLanguageWarning = True
         
+        self.deeplLabel.setText(self.tr(
+            """<p>A key is <b>required</b> to use this service."""
+            """ <a href="{0}">Get a commercial API key.</a></p>""").format(
+                TranslatorEngines.getKeyUrl("deepl")))
         self.googlev2Label.setText(self.tr(
             """<p>A key is <b>required</b> to use this service."""
             """ <a href="{0}">Get a commercial API key.</a></p>""").format(
@@ -76,6 +80,9 @@ class TranslatorPage(ConfigurationPageBase, Ui_TranslatorPage):
             self.languagesList.addItem(itm)
         self.languagesList.sortItems()
         
+        # DeepL settings
+        self.deeplKeyEdit.setText(
+            self.__plugin.getPreferences("DeeplKey"))
         # Google settings
         self.dictionaryCheckBox.setChecked(
             self.__plugin.getPreferences("GoogleEnableDictionary"))
@@ -109,6 +116,9 @@ class TranslatorPage(ConfigurationPageBase, Ui_TranslatorPage):
         self.__plugin.setPreferences(
             "EnabledLanguages", enabledLanguages)
         
+        # DeepL settings
+        self.__plugin.setPreferences(
+            "DeeplKey", self.deeplKeyEdit.text())
         # Google settings
         self.__plugin.setPreferences(
             "GoogleEnableDictionary", self.dictionaryCheckBox.isChecked())
