@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2008 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2008 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -56,7 +56,7 @@ class SearchReplaceWidget(QWidget):
             self.replaceHistory = vm.getSRHistory('replace')
             self.ui = Ui_ReplaceWidget()
             whatsThis = self.tr(
-                r"""<b>Find and Replace</b>
+                """<b>Find and Replace</b>
 <p>This dialog is used to find some text and replace it with another text.
 By checking the various checkboxes, the search can be made more specific.
 The search string might be a regular expression. In a regular expression,
@@ -67,7 +67,7 @@ special characters interpreted are:</p>
             from .Ui_SearchWidget import Ui_SearchWidget
             self.ui = Ui_SearchWidget()
             whatsThis = self.tr(
-                r"""<b>Find</b>
+                """<b>Find</b>
 <p>This dialog is used to find some text. By checking the various checkboxes,
 the search can be made more specific. The search string might be a regular
 expression. In a regular expression, special characters interpreted are:</p>
@@ -78,22 +78,22 @@ expression. In a regular expression, special characters interpreted are:</p>
             self.ui.wrapCheckBox.setChecked(True)
         
         whatsThis += self.tr(
-            r"""<table border="0">
+            """<table border="0">
 <tr><td><code>.</code></td><td>Matches any character</td></tr>
-<tr><td><code>\(</code></td><td>This marks the start of a region for tagging a
+<tr><td><code>(</code></td><td>This marks the start of a region for tagging a
 match.</td></tr>
-<tr><td><code>\)</code></td><td>This marks the end of a tagged region.
+<tr><td><code>)</code></td><td>This marks the end of a tagged region.
 </td></tr>
 <tr><td><code>\\n</code></td>
 <td>Where <code>n</code> is 1 through 9 refers to the first through ninth
 tagged region when replacing. For example, if the search string was
-<code>Fred\([1-9]\)XXX</code> and the replace string was <code>Sam\1YYY</code>,
-when applied to <code>Fred2XXX</code> this would generate <code>Sam2YYY</code>.
-</td></tr>
-<tr><td><code>\&lt;</code></td>
+<code>Fred([1-9])XXX</code> and the replace string was
+<code>Sam\\1YYY</code>, when applied to <code>Fred2XXX</code> this would
+generate <code>Sam2YYY</code>.</td></tr>
+<tr><td><code>\\&lt;</code></td>
 <td>This matches the start of a word using Scintilla's definitions of words.
 </td></tr>
-<tr><td><code>\&gt;</code></td>
+<tr><td><code>\\&gt;</code></td>
 <td>This matches the end of a word using Scintilla's definition of words.
 </td></tr>
 <tr><td><code>\\x</code></td>
@@ -548,7 +548,8 @@ and so on.</td></tr>
                 self.ui.wordCheckBox.isChecked(),
                 self.ui.wrapCheckBox.isChecked(),
                 not backwards,
-                line, index)
+                line, index,
+                posix=self.ui.regexpCheckBox.isChecked())
         
         if ok and self.ui.selectionCheckBox.isChecked():
             lineFrom, indexFrom, lineTo, indexTo = aw.getSelection()
@@ -599,7 +600,8 @@ and so on.</td></tr>
                         self.ui.wordCheckBox.isChecked(),
                         self.ui.wrapCheckBox.isChecked(),
                         not backwards,
-                        line, index)
+                        line, index,
+                        posix=self.ui.regexpCheckBox.isChecked())
                     if ok:
                         lineFrom, indexFrom, lineTo, indexTo = \
                             aw.getSelection()
@@ -625,7 +627,8 @@ and so on.</td></tr>
                         self.ui.wordCheckBox.isChecked(),
                         self.ui.wrapCheckBox.isChecked(),
                         not backwards,
-                        line, index)
+                        line, index,
+                        posix=self.ui.regexpCheckBox.isChecked())
                     if ok:
                         lineFrom, indexFrom, lineTo, indexTo = \
                             aw.getSelection()
@@ -842,7 +845,8 @@ and so on.</td></tr>
             self.ui.regexpCheckBox.isChecked(),
             self.ui.caseCheckBox.isChecked(),
             self.ui.wordCheckBox.isChecked(),
-            False, True, line, index)
+            False, True, line, index,
+            posix=self.ui.regexpCheckBox.isChecked())
         
         if ok and self.ui.selectionCheckBox.isChecked():
             lineFrom, indexFrom, lineTo, indexTo = aw.getSelection()
@@ -881,7 +885,8 @@ and so on.</td></tr>
                         self.ui.regexpCheckBox.isChecked(),
                         self.ui.caseCheckBox.isChecked(),
                         self.ui.wordCheckBox.isChecked(),
-                        False, True, line, index)
+                        False, True, line, index,
+                        posix=self.ui.regexpCheckBox.isChecked())
                     if ok:
                         lineFrom, indexFrom, lineTo, indexTo = \
                             aw.getSelection()
