@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -12,13 +12,13 @@ from __future__ import unicode_literals
 import os
 import sys
 
-from PyQt5.QtCore import QTranslator, QLocale, QLibraryInfo, QDir, \
-    QTextCodec, qVersion
+from PyQt5.QtCore import QTranslator, QLocale, QLibraryInfo, QDir, QTextCodec
 from PyQt5.QtWidgets import QApplication
 
 from E5Gui.E5Application import E5Application
 
 import Globals
+from Globals import qVersionTuple
 
 import UI.PixmapCache
 
@@ -60,7 +60,7 @@ def version(appinfo):
           """\n"""
           """{description}\n"""
           """\n"""
-          """Copyright (c) 2002 - 2017 Detlev Offenbach"""
+          """Copyright (c) 2002 - 2019 Detlev Offenbach"""
           """ <detlev@die-offenbachs.de>\n"""
           """This is free software; see LICENSE.GPL3 for copying"""
           """ conditions.\n"""
@@ -164,7 +164,7 @@ def loadTranslators(qtTransDir, app, translationFiles=()):
     
     global loaded_translators
     
-    if qVersion() < "5.0.0":
+    if qVersionTuple() < (5, 0, 0):
         # set the default encoding for tr()
         QTextCodec.setCodecForTr(QTextCodec.codecForName("utf-8"))
     
