@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2017 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -313,7 +313,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
         if self.filterCheckBox.isChecked():
             fileFilter = self.filterEdit.text()
             fileFilterList = \
-                ["^{0}$".format(filter.replace(".", "\.").replace("*", ".*"))
+                ["^{0}$".format(filter.replace(".", r"\.").replace("*", ".*"))
                  for filter in fileFilter.split(";")]
             filterRe = re.compile("|".join(fileFilterList))
         
@@ -341,7 +341,7 @@ class FindFileDialog(QDialog, Ui_FindFileDialog):
                 if self.sourcesCheckBox.isChecked():
                     filters.extend(
                         ["^{0}$".format(
-                            assoc.replace(".", "\.").replace("*", ".*"))
+                            assoc.replace(".", r"\.").replace("*", ".*"))
                          for assoc in list(
                              Preferences.getEditorLexerAssocs().keys())
                          if assoc not in self.formsExt + self.interfacesExt +
