@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2016 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2016 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -50,16 +50,15 @@ class NetworkManager(QNetworkAccessManager):
         """
         super(NetworkManager, self).__init__(parent)
         
-        if not WebBrowserWindow._fromEric:
-            from E5Network.E5NetworkProxyFactory import E5NetworkProxyFactory
-            
-            self.__proxyFactory = E5NetworkProxyFactory()
-            if Preferences.getUI("UseSystemProxy"):
-                QNetworkProxyFactory.setUseSystemConfiguration(True)
-            else:
-                QNetworkProxyFactory.setApplicationProxyFactory(
-                    self.__proxyFactory)
-                QNetworkProxyFactory.setUseSystemConfiguration(False)
+        from E5Network.E5NetworkProxyFactory import E5NetworkProxyFactory
+        
+        self.__proxyFactory = E5NetworkProxyFactory()
+        if Preferences.getUI("UseSystemProxy"):
+            QNetworkProxyFactory.setUseSystemConfiguration(True)
+        else:
+            QNetworkProxyFactory.setApplicationProxyFactory(
+                self.__proxyFactory)
+            QNetworkProxyFactory.setUseSystemConfiguration(False)
         
         self.languagesChanged()
         
@@ -340,13 +339,12 @@ class NetworkManager(QNetworkAccessManager):
         """
         self.__interceptor.preferencesChanged()
             
-        if not WebBrowserWindow._fromEric:
-            if Preferences.getUI("UseSystemProxy"):
-                QNetworkProxyFactory.setUseSystemConfiguration(True)
-            else:
-                QNetworkProxyFactory.setApplicationProxyFactory(
-                    self.__proxyFactory)
-                QNetworkProxyFactory.setUseSystemConfiguration(False)
+        if Preferences.getUI("UseSystemProxy"):
+            QNetworkProxyFactory.setUseSystemConfiguration(True)
+        else:
+            QNetworkProxyFactory.setApplicationProxyFactory(
+                self.__proxyFactory)
+            QNetworkProxyFactory.setUseSystemConfiguration(False)
     
     def createRequest(self, op, request, data):
         """
