@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 
 """
@@ -14,6 +14,9 @@ of the IDE and starts the Qt event loop.
 from __future__ import unicode_literals
 
 import sys
+import os
+
+originalPathString = os.getenv("PATH")
 
 # generate list of arguments to be remembered for a restart
 restartArgsList = ["--no-splash", "--plugin", "--debug", "--config",
@@ -41,7 +44,6 @@ try:
 except AttributeError:
     pass
 
-import os
 import traceback
 import time
 import logging
@@ -396,7 +398,8 @@ def main():
     splash.showMessage(
         QCoreApplication.translate("eric6", "Generating Main Window..."))
     mainWindow = UserInterface(app, loc, splash, pluginFile, disabledPlugins,
-                               noopen, nocrash, disablecrash, restartArgs)
+                               noopen, nocrash, disablecrash, restartArgs,
+                               originalPathString)
     app.lastWindowClosed.connect(app.quit)
     mainWindow.show()
     
