@@ -1,7 +1,7 @@
 #!/usr/bin/env python2
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2002 - 2018 Detlev Offenbach <detlev@die-offenbachs.de>
+# Copyright (c) 2002 - 2019 Detlev Offenbach <detlev@die-offenbachs.de>
 #
 # This is the install script for eric6.
 
@@ -548,7 +548,12 @@ def cleanupSource(dirName):
                  if fnmatch.fnmatch(f, "*.pyc")]:
         os.remove(os.path.join(dirName, name))
     
-    # step 3: descent into subdirectories and delete them if empty
+    # step 3: delete *.orig files
+    for name in [f for f in os.listdir(dirName)
+                 if fnmatch.fnmatch(f, "*.orig")]:
+        os.remove(os.path.join(dirName, name))
+    
+    # step 4: descent into subdirectories and delete them if empty
     for name in os.listdir(dirName):
         name = os.path.join(dirName, name)
         if os.path.isdir(name):
@@ -1104,10 +1109,26 @@ def createWindowsLinks():
         # check, if pywin32 is available
         from win32com.client import Dispatch    # __IGNORE_WARNING__
     except ImportError:
-        print(
-            "\nThe Python package 'pywin32' is not installed. Desktop and"
-            " Start Menu entries will not be created."
+        installed = pipInstall(
+            "pywin32",
+            "\nThe Python package 'pywin32' could not be imported."
         )
+        if installed:
+            # create the links via an external script to get around some
+            # startup magic done by pywin32.pth
+            args = [
+                sys.executable,
+                os.path.join(os.path.dirname(__file__),
+                             "create_windows_links.py"),
+            ]
+            if includePythonVariant:
+                args.append("-y")
+            subprocess.call(args)
+        else:
+            print(
+                "\nThe Python package 'pywin32' is not installed. Desktop and"
+                " Start Menu entries will not be created."
+            )
         return
     
     regPath = "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer" + \
@@ -1447,6 +1468,15 @@ def doDependancyChecks():
         exit(1)
     print("Found {0}".format(pyuic))
     
+    if pyqtVariant != "PyQt4":
+        try:
+            from PyQt5 import QtWebEngineWidgets    # __IGNORE_WARNING__
+        except ImportError as msg:
+            installed = pipInstall(
+                "PyQtWebEngine",
+                "PyQtWebEngine could not be detected.\nError: {0}".format(msg)
+            )
+    
     try:
         if pyqtVariant == "PyQt4":
             from PyQt4 import Qsci      # __IGNORE_WARNING__
@@ -1821,7 +1851,8 @@ def main(argv):
     """
     The main function of the script.
 
-    @param argv the list of command line arguments.
+    @param argv list of command line arguments
+    @type list of str
     """
     import getopt
 
@@ -1907,6 +1938,18 @@ def main(argv):
     
     infoName = ""
     installFromSource = not os.path.isdir(sourceDir)
+    
+    # check dependencies
+    if depChecks:
+        doDependancyChecks()
+    
+    # cleanup source if installing from source
+    if installFromSource:
+        print("Cleaning up source ...")
+        sourceDir = os.path.dirname(__file__) or "."
+        cleanupSource(sourceDir)
+        print()
+    
     if installFromSource:
         sourceDir = os.path.dirname(__file__) or "."
         configName = os.path.join(sourceDir, "eric6config.py")
@@ -1918,9 +1961,6 @@ def main(argv):
     if len(cfg) == 0:
         createInstallConfig()
     
-    if depChecks:
-        doDependancyChecks()
-    
     # get rid of development config file, if it exists
     try:
         if installFromSource:
@@ -1932,12 +1972,6 @@ def main(argv):
     except EnvironmentError:
         pass
     
-    # cleanup source if installing from source
-    if installFromSource:
-        print("Cleaning up source ...")
-        cleanupSource(sourceDir)
-        print()
-    
     # cleanup old installation
     print("Cleaning up old installation ...")
     try:
